from concurrent.futures import ThreadPoolExecutor as TPE
from os.path import getsize
from queue import Queue
from time import sleep

from watchdog.events import PatternMatchingEventHandler
from watchdog.observers import Observer

from cast_convert.preferences import FILESIZE_CHECK_WAIT
from .convert import convert_video
from .media_info import is_video


def wait_for_stable_size(filename: str, wait: float = FILESIZE_CHECK_WAIT, previous: int = -1):
    while True:
        sleep(wait)

        filesize = getsize(filename)

        if filesize == previous:
            return filesize

        previous = filesize


def consume_video_queue(queue: Queue, debug: bool = False):
    while True:
        filename = queue.get()

        if debug:
            print("Getting filesize")

        filesize = wait_for_stable_size(filename)
        file_is_vid = is_video(filename)

        if debug:
            print(filesize)
            print(file_is_vid)

        if file_is_vid:
            print(convert_video(filename))

        else:
            print(filename, 'not video')


class AddedFileHandler(PatternMatchingEventHandler):
    def __init__(self, queue: Queue, *args, debug: bool=False, **kwargs):
        super().__init__(*args, **kwargs)
        self.queue = queue
        self.debug = debug

    def on_moved(self, event):
        if event.is_directory:
            return

        if self.debug:
            print(event)

        self.queue.put(event.dest_path)

    def on_created(self, event):
        if event.is_directory:
            return

        if self.debug:
            print(event)

        self.queue.put(event.src_path)


def watch_directory(directory: str, ignore_patterns: tuple=tuple(), debug: bool=True):
    observer = Observer()
    queue = Queue()
    handler = AddedFileHandler(queue, ignore_patterns=ignore_patterns, debug=debug)

    thread_pool = TPE(1)
    converter_future = thread_pool.submit(consume_video_queue, queue)

    print("Watching %s for new videos..." % directory)

    observer.schedule(handler, directory, recursive=True)
    observer.start()

    try:
        while True:
            sleep(1)

    except KeyboardInterrupt:
        observer.stop()

    try:
        observer.join()
        queue.join()
        converter_future.cancel()
        thread_pool.shutdown(wait=0.1)

    except KeyboardInterrupt:
        exit()




import click

from .watch import watch_directory
from .convert import convert_video, need_to_transcode, get_ffmpeg_cmd
from .media_info import get_transcode_info
from .preferences import THREADS


@click.group(help="Convert and inspect video for Chromecast compatibility")
def cmd(debug: bool=False):
    pass


@click.command(help="Generate ffmpeg conversion command.")
@click.argument("filename")
def get_cmd(filename: str):
    print(get_ffmpeg_cmd(filename))


@click.command(help="Convert video to Chromecast compatible encodings and container")
@click.argument("filename")
@click.option("-t", "--threads", default=THREADS,  type=click.INT,
              help="Count of threads for ffmpeg to use. Default: %s" % THREADS)
def convert(filename: str, threads: int):
    print("%s -> %s" % (filename, convert_video(filename, threads)))


@click.command(help="Inspect video for transcoding options")
@click.argument("filename")
def inspect(filename: str):
    info = get_transcode_info(filename)

    if need_to_transcode(info):
        print("Transcode video to %s" % str(info))

    else:
        print("No need to transcode", filename)


@click.command(help="Watch directory and convert newly added videos")
@click.option('-i', '--ignore',
              help="File pattern to ignore. Flag can be used many times.",
              multiple=True)
@click.argument("directory")
def watch(directory: str, ignore: tuple):
    if ignore:
        print("Ignoring", ', '.join(ignore) + '...')

    watch_directory(directory, ignore_patterns=ignore)


cmd.add_command(get_cmd)
cmd.add_command(convert)
cmd.add_command(inspect)
cmd.add_command(watch)


if __name__ == "__main__":
    cmd()

